/*
 * gbakeys.c
 * keyboard emulation...
 */

#include <stdlib.h>
#include <math.h>
#include "gbakeys.h"
#include "curses.h"
#include "keys.h"
#include "keypad.h"
#include "screenmode.h"
#include "gbaui.h"

#include "rogue.h"

#define ALL_KEYS (KEY_A|KEY_B|KEY_SELECT|KEY_START|KEY_UP|KEY_DOWN|KEY_LEFT|KEY_RIGHT|KEY_L|KEY_R)
#define KEY_FIRST_PRESS  4
#define KEY_REPEAT 30

void gba_set_scale(int x, int y);
int gbagetinvchar(const char descs[MAX_PACK_COUNT+1][DCOLS], int count) ;
int min(int a, int b) {
  return a < b ? a : b;
}

keys_t g_keypad;

// update the 'virtual key' arrays
void update_keys()
{
  // get keypresses from hardware
  UpdateKeys();
  // make set == on
  int i;
  for (i = 0; i < 10; i++)
  {
    if ( KeyPressed (1<<i) )
    {
      g_keypad.key_held[i]++;
      if ( g_keypad.key_held[i] > KEY_REPEAT) {
        g_keypad.key_held[i]--;
      }
    } else {
      g_keypad.key_held[i] = 0;
    }
    if (g_keypad.key_held[i] == KEY_FIRST_PRESS || g_keypad.key_held[i] >= KEY_REPEAT)
    {
      g_keypad.key[i] = 1;
    }  else {
      g_keypad.key[i] = 0;
    }
  }
}
void init_keypresses() {
  int i;
  for (i = 0; i < 10; i++)
  {
    g_keypad.key[i] = 0;
    g_keypad.key_held[i] = 0;
  }
}

typedef struct {
  int dx;
  int dy;
  int rx;
  int ry;
  int curr_x;
  int curr_y;
  int x;
  int y;
} gba_rot_t;
gba_rot_t g_rotbg={0,0,0,0};

// this is called whenever we want a "character", but has no context (nor should context be needed)

// problems here are: how to integrate quaffing, eating, searching, etc
// how to get at our inventory
// how to accomodate the second key or direction or whatever when eating.... 
/*
 * solution 1: 
 *   'get direction' getchar routine just to get movements
 *   'get inventory' getchar routine to get an item from inventory
 *                   this would require a list of items passed in (item, key) which
 *                   the routine displays, handles menu selection, etc then returns choice as char
 *   'get action' getchar routine : would first have to be activated by another key, eg Start 
 *                and only when in movement mode. Then would show a list of possible actions 
 *                (as words), handle menu selection and return the selected action as 
 *                the char ('q' for quaff, 'e', 'w' etc)                 
 *
 * soution 2:
 *   have a generic getchar that internally manages 'virtual keys mode' and 'movement mode'
 *   this is the solution that AGB_Rogue et al take: when Start is pressed, a virtual keyboard
 *   is shown and up, down, etc move around and seelct letters
 *   when not in VK mode, up, down, etc act as 'vim keys'
 *   An extra complication here is how to show the inventory, as it can't be shown in the default font!
 *   Would have to write the inventory displayer, but not the menu handler and would not have to worry about
 *   checking all getchar calls and changing them to special custom ones
 *
 * solution 3:
 *   as for 2, but show the inventory as 'item sprites' (max 25! perfect) and only show text when highlighting them
 */

#define EXTENDED_OPTIONS_COUNT   20
/* 
  format is :
   " XY blah"
    |||| |
    |||| +--- description
    |||+----- single space
    ||+------ glyph shwon in the list
    |+------- key pressed when selected
    +-------- single space
*/
const char g_extended_options[MAX_PACK_COUNT+1][DCOLS] = {
  " .r Rest",
  " ss Search",
  " q! Quaff",
  " eh Eat",
  " fl Fight",
  " r? Read",
  " P= Put on ring",
  " <% Ascend level",
  " ^^ Identify trap",
  " T[ Take off",
  " wp Wield weapon",
  " z/ Zap wand",
  " i@ Inventory",
  " R= Remove a ring",
  " >% Descend level",
  " W] Wear armour",
  " td Throw something",
  " Fl Fight to the death",
  " d} Drop",
//  " ]] List armour",
//  " == List rings",
//  " )) List weapons",
};

#define KEY_L_DBL_CLK  25
int g_last_key = ' ';
int gbagetchar(void)
{ 
  int key_pressed = 0;
  int key_l = 0;
  while (!key_pressed) {
    wait_vsync_int();
    update_keys();

    if (g_keypad.key[GBA_VK_L]) {
      if (key_l && key_l < KEY_L_DBL_CLK) {
        // we have double clicked
        if (g_rotbg.rx == 256) {
          gba_set_scale(256+256+180,256+108);
        } else {
          gba_set_scale(256,256);
        } 
      } else {
        key_l = KEY_L_DBL_CLK;
      }
    } else {
      if (key_l) {
        key_l--;
      }
    }

    
    if (g_keypad.key_held[GBA_VK_L]) {
      // scroll the screen
      if (g_keypad.key[GBA_VK_RIGHT]) {
        curscr->bg.DX+= 256;
      }
      if (g_keypad.key[GBA_VK_LEFT]) {
        curscr->bg.DX-= 256;
      }
      if (g_keypad.key[GBA_VK_DOWN]) {
        curscr->bg.DY+= 256;
      }
      if (g_keypad.key[GBA_VK_UP]) {
        curscr->bg.DY-= 256;
      }
      UpdateBackground(&curscr->bg);
    } else {
      // need to somehow get diagonals in here...
      // plus a way to change A / B to other keys!
      if (g_keypad.key[GBA_VK_A]) {
        key_pressed = ' ';
        break;
      }
#define DIAGONAL_CHECK(VK) (g_keypad.key_held[VK] && ((g_keypad.key_held[VK]<KEY_FIRST_PRESS) || g_keypad.key[VK]))
      if (DIAGONAL_CHECK(GBA_VK_UP) && DIAGONAL_CHECK(GBA_VK_LEFT)) {
        key_pressed = ROGUE_KEY_NORTHWEST;
        g_keypad.key_held[GBA_VK_UP] = KEY_FIRST_PRESS+2;
        g_keypad.key_held[GBA_VK_LEFT] = KEY_FIRST_PRESS+2;
      }
      if (DIAGONAL_CHECK(GBA_VK_UP) && DIAGONAL_CHECK(GBA_VK_RIGHT)) {
        key_pressed = ROGUE_KEY_NORTHEAST;
        g_keypad.key_held[GBA_VK_UP] = KEY_FIRST_PRESS+2;
        g_keypad.key_held[GBA_VK_RIGHT] = KEY_FIRST_PRESS+2;
      }
      if (DIAGONAL_CHECK(GBA_VK_DOWN) && DIAGONAL_CHECK(GBA_VK_LEFT)) {
        key_pressed = ROGUE_KEY_SOUTHWEST;
        g_keypad.key_held[GBA_VK_DOWN] = KEY_FIRST_PRESS+2;
        g_keypad.key_held[GBA_VK_LEFT] = KEY_FIRST_PRESS+2;
      }
      if (DIAGONAL_CHECK(GBA_VK_DOWN) && DIAGONAL_CHECK(GBA_VK_RIGHT)) {
        key_pressed = ROGUE_KEY_SOUTHEAST;
        g_keypad.key_held[GBA_VK_DOWN] = KEY_FIRST_PRESS+2;
        g_keypad.key_held[GBA_VK_RIGHT] = KEY_FIRST_PRESS+2;
      }

      if (!key_pressed) {
        if ( (g_keypad.key_held[GBA_VK_UP] || g_keypad.key_held[GBA_VK_DOWN]) &&
            (g_keypad.key_held[GBA_VK_LEFT] || g_keypad.key_held[GBA_VK_RIGHT]) ) {
          continue;
        }
        if (!g_keypad.key_held[GBA_VK_B]) {
          if (g_keypad.key[GBA_VK_UP]) {
            key_pressed = ROGUE_KEY_NORTH;
          }
          if (g_keypad.key[GBA_VK_DOWN]) {
            key_pressed = ROGUE_KEY_SOUTH;
          }
          if (g_keypad.key[GBA_VK_LEFT]) {
            key_pressed = ROGUE_KEY_WEST;
          }
          if (g_keypad.key[GBA_VK_RIGHT]) {
            key_pressed = ROGUE_KEY_EAST;
          }
        }
      }
      if (key_pressed) {
        if (g_keypad.key_held[GBA_VK_R]) {
          // control pressed
          key_pressed -= ('a'-1);
        }
        break;
      }
      if (g_keypad.key[GBA_VK_SELECT]) {
        key_pressed = g_last_key;
        break;
      }
      if (g_keypad.key[GBA_VK_B]) {
        key_pressed = ROGUE_KEY_CANCEL;
        break;
      }
      if (g_keypad.key[GBA_VK_START]) {
        // this should show an extended options menu with quaff, etc
        gbaui_inventory(g_extended_options , EXTENDED_OPTIONS_COUNT);
        key_pressed = gbagetinvchar(g_extended_options , EXTENDED_OPTIONS_COUNT);
        void gbaui_clrinv(int count);
        gbaui_clrinv(EXTENDED_OPTIONS_COUNT);
        //gbaprintf("returned #%c# %x\n", key_pressed,key_pressed);
        if (key_pressed == (int)ROGUE_KEY_CANCEL) {
          key_pressed = 0;
        } else {
          g_last_key = key_pressed;
          break;
        }
      }
    }
  }
  return key_pressed;
}

void center_screen()
{
  curscr->bg.DX = 20*256;
  curscr->bg.DY = 256;
  UpdateBackground(&curscr->bg);

}

#define MAX_X   (DCOLS - 15)
#define MAX_Y   (DROWS - 19)

#define MAX_DX  10
#define MAX_DY  5

void gba_scroll_player(int x, int y) 
{
  
  // store the current position - this needs to be seperate to the 
  // actual scroll position so that we can stop the scrolling going negative
/*  static int curr_x = -100;
  static int curr_y = -100;*/
  // draw '@' at x, y (absolute)
  // deal with tile coords, convert to pixels when writing DX DY
  int new_x, new_y;
  g_rotbg.x = x;
  g_rotbg.y = y;
  
  // check that the player is not on the edge of the screen
  new_x = x - 15;  // 30 tiles wide, we should be in the middle
  new_y = y - 10;  // 20 tiles high,   --       "         --
  // need to see if new_x, new_y is in the current 'window'
  int dx = new_x - g_rotbg.curr_x;
  int dy = new_y - g_rotbg.curr_y;

  /*gbaprintf("x %d y %d dx %d dy %d newx %d newy %d currx %d curry %d\n", 
      x, y, dx, dy, new_x, new_y, curr_x, curr_y);*/
  
  if (abs(dx) > MAX_DX || abs(dy) > MAX_DY) {
    wait_vsync_int();
    // store the 'real' current values
    g_rotbg.curr_x = new_x;
    g_rotbg.curr_y = new_y;
    if (new_x < 0) { new_x = 0; }
    if (new_y < 0) { new_y = 0; }
    if (new_x > MAX_X) { new_x = MAX_X; }
    if (new_y > MAX_Y) { new_y = MAX_Y; }
    g_rotbg.dx = 8*(new_x)*256;
    g_rotbg.dy = 8*(new_y)*256;
  } 

  if (g_rotbg.rx == 256) {
    curscr->bg.DX = g_rotbg.dx;
    curscr->bg.DY = g_rotbg.dy;
  } else {
    curscr->bg.DX = 0;
    curscr->bg.DY = 0;
  }
}

// set the rot bg scale
void gba_set_scale(int x, int y) {
/*if (x > 256 || x < -79) {
    return ;
  }*/
  
  g_rotbg.rx = x;
  g_rotbg.ry = y;
  
  curscr->bg.PA = g_rotbg.rx;
  curscr->bg.PB = 0;
  curscr->bg.PC = 0;
  curscr->bg.PD = g_rotbg.ry;

  if (g_rotbg.rx == 256) {
    g_rotbg.curr_x = -100;
    g_rotbg.curr_y = -100;
    gba_scroll_player(g_rotbg.x, g_rotbg.y);
    curscr->bg.DX = g_rotbg.dx;
    curscr->bg.DY = g_rotbg.dy;
  } else {
    curscr->bg.DX = 0;
    curscr->bg.DY = 0;
  }
  /*
  curscr->bg.PA = (256*256)/(240+scale);
  curscr->bg.PB = 0;
  curscr->bg.PC = 0;
  curscr->bg.PD = (192*256)/(160+scale*2);
  */
}

int gbagetinvchar(const char descs[MAX_PACK_COUNT+1][DCOLS], int count) 
{
  count--;
  static int selected = 0;
  if (selected >= count)
    selected = 0;
  hilite_selection(selected, &descs[selected][3]);
  for (;;) {
    wait_vsync_int();
    update_keys();
    if (g_keypad.key[GBA_VK_RIGHT]) {
      // increase selection
      selected++;
      if (selected >= count) {
        selected = 0;
      }
      hilite_selection(selected, &descs[selected][3]);
    }
    if (g_keypad.key[GBA_VK_LEFT]) {
      // increase selection
      selected--;
      if (selected < 0) {
        selected = count-1;
      }
      hilite_selection(selected, &descs[selected][3]);
    }
    
    if (g_keypad.key[GBA_VK_A] || g_keypad.key[GBA_VK_B]) {
      break;
    }
  }
  hilite_selection(-1,0);
  if (g_keypad.key[GBA_VK_A]) {
    return descs[selected][1];
  }
  if (g_keypad.key[GBA_VK_B]) {
    return ROGUE_KEY_CANCEL;
  }
  return ' ';
}

int gbaget_l_or_r()
{
  for (;;) {
    wait_vsync_int();
    update_keys();
    if (g_keypad.key[GBA_VK_RIGHT]) 
      return 'r';
    if (g_keypad.key[GBA_VK_LEFT]) 
      return 'l';
    if (g_keypad.key[GBA_VK_B]) 
      return ROGUE_KEY_CANCEL;

  }
}

