#include "rogue.h"
#include "curses.h"
#include "sprite.h"
#include "text.h"

#define SPRITE_LEVEL      16
#define SPRITE_GOLD       26
#define SPRITE_HP         14
#define SPRITE_STRENGTH   10
#define SPRITE_ARMOR      18
#define SPRITE_EXP        144
#define SPRITE_HUNGER     150


#define SPRITE_LEVEL_X      0
#define SPRITE_GOLD_X       (SPRITE_LEVEL_X+3)
#define SPRITE_HP_X         (SPRITE_GOLD_X+6)
#define SPRITE_STRENGTH_X   (SPRITE_HP_X+7)
#define SPRITE_ARMOR_X      (SPRITE_STRENGTH_X+6)

#define SPRITE_EXP_X        (0)
#define SPRITE_HUNGER_X     (SPRITE_EXP_X+12)

#define SPRITE_LEVEL_Y      18
#define SPRITE_GOLD_Y       18
#define SPRITE_HP_Y         18
#define SPRITE_STRENGTH_Y   18
#define SPRITE_ARMOR_Y      18

#define SPRITE_EXP_Y        19
#define SPRITE_HUNGER_Y     19



#define SPRITE_LEVEL_ID      127
#define SPRITE_GOLD_ID      (SPRITE_LEVEL_ID-1)
#define SPRITE_HP_ID        (SPRITE_LEVEL_ID-2)
#define SPRITE_STRENGTH_ID  (SPRITE_LEVEL_ID-3)
#define SPRITE_ARMOR_ID     (SPRITE_LEVEL_ID-4)
#define SPRITE_EXP_ID       (SPRITE_LEVEL_ID-5)
#define SPRITE_HUNGER_ID    (SPRITE_LEVEL_ID-6)
#define SPRITE_INVENTORY_ID      (SPRITE_HUNGER_ID-1)

OAMEntry sprites[128] IN_EWRAM;
void init_sprite8(int index, int priority, int oam) {
  OBJPaletteMem[27] = 0;
  sprites[index].attribute0 = SQUARE|COLOR_256; 
  sprites[index].attribute1 = SIZE_8; 
  sprites[index].attribute2 = PRIORITY(priority) | (oam);
}
// this requires that the 'curses' also loads its code to sprite ram
void add_label(int stat)
{
  switch (stat) {
    case STAT_LEVEL:
      init_sprite8(SPRITE_LEVEL_ID, 0, SPRITE_LEVEL);
      MoveSprite(&sprites[SPRITE_LEVEL_ID], SPRITE_LEVEL_X*8, SPRITE_LEVEL_Y*8);
      break;
    case STAT_GOLD:
      init_sprite8(SPRITE_GOLD_ID, 0, SPRITE_GOLD);
      MoveSprite(&sprites[SPRITE_GOLD_ID], SPRITE_GOLD_X*8, SPRITE_GOLD_Y*8);
      break;
    case STAT_HP:
      init_sprite8(SPRITE_HP_ID, 0, SPRITE_HP);
      MoveSprite(&sprites[SPRITE_HP_ID], SPRITE_HP_X*8, SPRITE_HP_Y*8);
      break;
    case STAT_STRENGTH:
      init_sprite8(SPRITE_STRENGTH_ID, 0, SPRITE_STRENGTH);
      MoveSprite(&sprites[SPRITE_STRENGTH_ID], SPRITE_STRENGTH_X*8, SPRITE_STRENGTH_Y*8);
      break;
    case STAT_ARMOR:
      init_sprite8(SPRITE_ARMOR_ID, 0, SPRITE_ARMOR);
      MoveSprite(&sprites[SPRITE_ARMOR_ID], SPRITE_ARMOR_X*8, SPRITE_ARMOR_Y*8);
      break;
    case STAT_EXP:
      init_sprite8(SPRITE_EXP_ID, 0, SPRITE_EXP);
      MoveSprite(&sprites[SPRITE_EXP_ID], SPRITE_EXP_X*8, SPRITE_EXP_Y*8);
      break;
    case STAT_HUNGER:
      init_sprite8(SPRITE_HUNGER_ID, 0, SPRITE_HUNGER);
      MoveSprite(&sprites[SPRITE_HUNGER_ID], SPRITE_HUNGER_X*8, SPRITE_HUNGER_Y*8);
      break;
  }
  CopyOAM();
}
void remove_label(int stat)
{
  switch (stat) {
    case STAT_LEVEL:
      remove_sprite(SPRITE_LEVEL_ID);
      break;
    case STAT_GOLD:
      remove_sprite(SPRITE_GOLD_ID);
      break;
    case STAT_HP:
      remove_sprite(SPRITE_HP_ID);
      break;
    case STAT_STRENGTH:
      remove_sprite(SPRITE_STRENGTH_ID);
      break;
    case STAT_ARMOR:
      remove_sprite(SPRITE_ARMOR_ID);
      break;
    case STAT_EXP:
      remove_sprite(SPRITE_EXP_ID);
      break;
    case STAT_HUNGER:
      remove_sprite(SPRITE_HUNGER_ID);
      break;
  }
  CopyOAM();
}


void clear_stats()
{
  statsclrtoeol(SPRITE_LEVEL_Y);
}

void add_stat(int stat, const char * text) 
{
  int x = 0;
  int y = 0;
  switch (stat) {
    case STAT_LEVEL:
      x = SPRITE_LEVEL_X+1;
      y = SPRITE_LEVEL_Y;
      break;
    case STAT_GOLD:
      x = SPRITE_GOLD_X+1;
      y = SPRITE_GOLD_Y;
      break;
    case STAT_HP:
      x = SPRITE_HP_X+1;
      y = SPRITE_HP_Y;
      break;
    case STAT_STRENGTH:
      x = SPRITE_STRENGTH_X+1;
      y = SPRITE_STRENGTH_Y;
      break;
    case STAT_ARMOR:
      x = SPRITE_ARMOR_X+1;
      y = SPRITE_ARMOR_Y;
      break;
    case STAT_EXP:
      x = SPRITE_EXP_X+1;
      y = SPRITE_EXP_Y;
      break;
    case STAT_HUNGER:
      x = SPRITE_HUNGER_X+1;
      y = SPRITE_HUNGER_Y;
      break;
  }

  statstr(text,x,y);
}

void init_sprites() 
{
  int i;
  for (i = 0; i < 128; i++)
    sprites[i].attribute0 = SIZE_DOUBLE; 
  CopyOAM();
}

void deactivate_sprites(int start, int end) {
  int i;
  for (i = start; i < end; i++) {
    sprites[i].attribute0 |= SIZE_DOUBLE;
    sprites[i].attribute0 &= ~ROTATION_FLAG;
  }
}

void gbaui_inventory(const char descs[MAX_PACK_COUNT+1][DCOLS], int count)
{
  int i;
  int y = 28;
  count--; 
  for (i = 0; i < count; ++i) {
    init_sprite8(SPRITE_INVENTORY_ID-i, 0, (int)(descs[i][2] - ' '+3)*2);
    int x = (i)*8;
    MoveSprite(&sprites[SPRITE_INVENTORY_ID-i], x,y);
  }
  CopyOAM();
}

void gbaui_clrinv(int count)
{
  int i;
  for (i = 0; i < count; ++i) {
    sprites[SPRITE_INVENTORY_ID-i].attribute0 = SIZE_DOUBLE;
  }
  CopyOAM();
}

void  hilite_selection( int selected, const char * desc)
{

  messageclrtoeol();
  int i;
  for (i = SPRITE_INVENTORY_ID-30; i != SPRITE_INVENTORY_ID+1; i++) {
    sprites[i].attribute0 |= MODE_TRANSPARENT;
  }
  if (selected == -1)
    return;
  sprites[SPRITE_INVENTORY_ID-selected].attribute0 &= ~MODE_TRANSPARENT;
  messagestr(desc);
  REG_BLDMOD = BLDMOD_SECOND_BG0 |
               BLDMOD_SECOND_BG1 | BLDMOD_SECOND_BG2 | BLDMOD_SECOND_BD | 
               BLDMOD_MODE(0);
  REG_COLEV = 10 | ((5)<<8); // fade from all sprite (16|0) to all bg (0|16)
  CopyOAM();
}
