#include "gba.h"
#include "gbakeys.h"
#include "keypad.h"
#include "keyboard.h"
#include "text.h"
#include "sprite.h"
#include "screenmode.h"

#define VK_SELECT_REPEAT  30
#define VK_SPRITE_OAM (224*64)
#define VK_PALETTE (12)
#define VK_START_SPRITE (0)

int g_keyboard_position = 0;
typedef struct{
  unsigned char x;
  unsigned char y;
} coord_t;
static const coord_t keyboard_positions[] = {
  {10,20} ,
  {149,20} ,    // 9*9 = 81,  240 - 81 - 10 = 149
  {149,113} ,   // 3*9 = 27,  160 - 27 - 20 = 113
  {10,113} ,
};

int g_vk_hilite = 0;
int g_keyboard_str_index = 0;
char g_keyboard_str[9];

FunctionPtr keyboard_on_change_cb = 0;
FunctionPtr keyboard_enter_cb = 0;

void set_vk_callbacks(FunctionPtr on_change, FunctionPtr on_enter, int return_screen) 
{
  keyboard_on_change_cb = on_change;
  keyboard_enter_cb = on_enter;
}

void hilite_vk(int keyoff, int keyon) {
  keyoff -= 'A';
  keyon -= 'A';
  sprites[VK_START_SPRITE+keyoff].attribute2 &= ~PALETTE(15);
  sprites[VK_START_SPRITE+keyon].attribute2 &= ~PALETTE(15);
  sprites[VK_START_SPRITE+keyoff].attribute2 |= PALETTE(VK_PALETTE);
  sprites[VK_START_SPRITE+keyon].attribute2 |= PALETTE((VK_PALETTE+1));
  CopyOAM();
}
static const coord_t vk_pos[27] = {
  {0,1}, //a
  {1,1}, //b
  {2,1}, //c
  {3,1}, //d
  {4,1}, //e
  {5,1}, //f
  {6,1}, //g
  {7,1}, //h
  {8,1}, //i
  {0,2}, //j
  {1,2}, //k
  {2,2}, //l
  {3,2}, //m
  {4,2}, //n
  {5,2}, //o
  {6,2}, //p
  {7,2}, //q
  {8,2}, //r
  {0,3}, //s
  {1,3}, //t
  {2,3}, //u
  {3,3}, //v
  {4,3}, //w
  {5,3}, //x
  {6,3}, //y
  {7,3}, //z
  {8,3}, //?
};
void init_vk_sprites(int x_off, int y_off) {

  // move relative to the cursor position
  // if the cursor is on the left, draw the keyboard on the right
  OBJPaletteMem[VK_PALETTE*16+FONT_COL_INDEX] = RGB16(31,31,31);
  OBJPaletteMem[VK_PALETTE*16+FONT_COL_INDEX+1] = RGB16(0,11,21);
  OBJPaletteMem[(VK_PALETTE+1)*16+FONT_COL_INDEX] = RGB16(31,31,31);
  OBJPaletteMem[(VK_PALETTE+1)*16+FONT_COL_INDEX+1] = RGB16(31,21,11);
  int i;
  for (i = 0; i < 27; i++) {
    int idx = i+VK_START_SPRITE;
    sprites[idx].attribute0 = SQUARE|COLOR_16|( y_off + (vk_pos[i].y*9)); 
    sprites[idx].attribute1 = SIZE_8|(x_off + (vk_pos[i].x*9)); 
    sprites[idx].attribute2 = PRIORITY(0) | (VK_SPRITE_OAM/16+(i)) | PALETTE(VK_PALETTE);
  }
}

void load_vk(char enter) {
  // load the virtual keyboard to vram
  u32 * font = (u32*)&vwfont[32*('A'-' ')];
  u32 * vram = (u32*)&OAMData[VK_SPRITE_OAM];

  // DMA3_COPY(  &vwfont[32*('A'-' ')], 
  //        &OAMData[(100*32)], 
  //        ((26*16)|DMA_32NOW));
  int i, j, k;
  for (i = 0; i < 26; i++) {
    for (j = 0; j < 8; j++) {
      u32 pix = *font++;

      for (k = 0; k < 8; k++) {
        u32 mask = 0xf<<(k*4);
        if ((mask&pix)==0) {
          pix |= mask;
        }
      }
      *vram++= pix;
    }
  }
  // and the "enter" font
  font = (u32*)&vwfont[32*(enter-' ')];
  for (j = 0; j < 8; j++) {
    u32 pix = *font++;
    for (k = 0; k < 8; k++) {
      u32 mask = 0xf<<(k*4);
      if ((mask&pix)==0) {
        pix |= mask;
      }
    }
    *vram++= pix;
  }
  // init 26 sprites starting at sprite 100
  g_keyboard_position = 0;
  init_vk_sprites(keyboard_positions[g_keyboard_position].x, 
      keyboard_positions[g_keyboard_position].y);
  g_vk_hilite = 'A';
  hilite_vk('B',g_vk_hilite);
  g_keyboard_str_index = 0;
  g_keyboard_str[g_keyboard_str_index] = 0;
}

#define QUESTION_MARK '['

void virtual_keys(void) {

  // user_general_controls(VK_SELECT_REPEAT);
  update_keys();

  int init_hilite = g_vk_hilite;
  if (g_keypad.key[GBA_VK_RIGHT]) 
  {
    g_vk_hilite++;
    if ( (g_vk_hilite == 'J') ||
        (g_vk_hilite == 'S') ||
        (g_vk_hilite > QUESTION_MARK) ) {
      g_vk_hilite -= 9;
    }
  } 
  if (g_keypad.key[GBA_VK_LEFT]) 
  {
    g_vk_hilite--;
    if ( (g_vk_hilite == 'I') ||
        (g_vk_hilite == 'R') ||
        (g_vk_hilite <  'A') ) {
      g_vk_hilite += 9;
    }
  } 
  if (g_keypad.key[GBA_VK_UP]) 
  {
    g_vk_hilite-=9;
    if (g_vk_hilite < 'A') {
      g_vk_hilite+=27;
    }
  } 
  if (g_keypad.key[GBA_VK_DOWN]) 
  {
    g_vk_hilite+=9;
    if (g_vk_hilite > QUESTION_MARK) {
      g_vk_hilite-=27;
    }
  } 

  if (init_hilite != g_vk_hilite) {
    hilite_vk(init_hilite, g_vk_hilite);
  }

  if (g_keypad.key[GBA_VK_A]) 
  {
    // add letter
    if (g_vk_hilite != QUESTION_MARK && g_keyboard_str_index < 8) {
      g_keyboard_str[g_keyboard_str_index++] = g_vk_hilite;
      g_keyboard_str[g_keyboard_str_index] = 0;
      if (keyboard_on_change_cb)
        keyboard_on_change_cb();
      //update_find();
    } else if (g_vk_hilite == QUESTION_MARK) {
      // on q mark, search for planet name
      // find_named_planet();
      if (keyboard_enter_cb)
        keyboard_enter_cb();
      return;
    }
  }
  if (g_keypad.key[GBA_VK_B]) 
  {
    // add letter
    if (g_keyboard_str_index) {
      g_keyboard_str_index--;
      g_keyboard_str[g_keyboard_str_index] = 0;
      if (keyboard_on_change_cb)
        keyboard_on_change_cb();
    } 
  }
  if (KeyPressedNoBounce(KEY_START,3) ) {
    // search as well
    if (keyboard_enter_cb) {
      keyboard_enter_cb();
    }
  }
  if (g_keypad.key[GBA_VK_L]) 
  {
    g_keyboard_position--;
    g_keyboard_position&=3;
    init_vk_sprites(keyboard_positions[g_keyboard_position].x, 
        keyboard_positions[g_keyboard_position].y);
    hilite_vk(g_vk_hilite, g_vk_hilite);
  }
  if (g_keypad.key[GBA_VK_R]) { 
    g_keyboard_position++;
    g_keyboard_position&=3;
    init_vk_sprites(keyboard_positions[g_keyboard_position].x, 
        keyboard_positions[g_keyboard_position].y);
    hilite_vk(g_vk_hilite, g_vk_hilite);
  }
}


