/*
 * machdep.c
 *
 * This source herein may be modified and/or distributed by anybody who
 * so desires, with the following restrictions:
 *    1.)  No portion of this notice shall be removed.
 *    2.)  Credit shall not be taken for the creation of this source.
 *    3.)  This code is not to be traded, sold, or used for personal
 *         gain or profit.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "rogue.h"
#ifndef __thumb__
#include <pwd.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <termios.h>
#else
#include <string.h>
#include "pack.h"
#include "saving.h"
#include "gbakeys.h"
#include "screenmode.h"
#include "reset.h"
#include "keyboard.h"
#endif
#include "init.h"
#include "machdep.h"
#include "random.h"
#include "curses.h"

extern const char * const score_file;
/* md_init :
 *
 * This routine calls any machine dependent initialising routines
 *
 * Can be safely stubbed
 */
void md_init(void)
{
  extern void gba_scroll_player(int,int);
  register_scroll_cb('@', gba_scroll_player);
  init_keypresses();
  init_sprites();
  init_sound();
}

/* md_slurp:
 *
 * This routine throws away all keyboard input that has not
 * yet been read.  It is used to get rid of input that the user may have
 * typed-ahead.
 *
 * This function is not necessary, so it may be stubbed.  The might cause
 * message-line output to flash by because the game has continued to read
 * input without waiting for the user to read the message.  Not such a
 * big deal.
 */

void md_slurp(void)
{
// TODO: Check if this is really needed or else remove
#if 0
#ifdef UNIX_BSD4_2
	long ln;
	int i, n;
	ioctl(0, FIONREAD, &ln);
	n = (int) (stdin->_cnt + ln);

	for (i = 0; i < n; i++)
	{
		(void) getchar();
	}
#endif	/* UNIX_BSD4_2 */
#endif
}

/* md_control_keyboard():
 *
 * This routine is much like md_cbreak_no_echo_nonl() above.  It sets up the
 * keyboard for appropriate input.  Specifically, it prevents the tty driver
 * from stealing characters.  For example, ^Y is needed as a command
 * character, but the tty driver intercepts it for another purpose.  Any
 * such behavior should be stopped.  This routine could be avoided if
 * we used RAW mode instead of CBREAK.  But RAW mode does not allow the
 * generation of keyboard signals, which the program uses.
 *
 * The parameter 'mode' when true, indicates that the keyboard should
 * be set up to play rogue.  When false, it should be restored if
 * necessary.
 *
 * This routine is not strictly necessary and may be stubbed.  This may
 * cause certain command characters to be unavailable.
 */

void md_control_keyboard(int mode)
{
#if 0
	static boolean called_before = 0;
	static struct termios tc_orig;
	static struct termios tc_temp;

	signal(SIGTSTP,SIG_IGN);

	if (!called_before)
	{
		called_before = 1;
        tcgetattr(0, &tc_orig);	
	}
	tc_temp = tc_orig;

	if (!mode)
	{
		tc_temp.c_cc[VINTR] = -1;
		tc_temp.c_cc[VQUIT] = -1;
		tc_temp.c_cc[VERASE] = -1;
		tc_temp.c_cc[VKILL] = -1;
		tc_temp.c_cc[VEOF] = -1;
/* 		tc_temp.c_cc[VSWTCH] = -1; */
		tc_temp.c_cc[VSTART] = -1;
		tc_temp.c_cc[VSTOP] = -1;
		tc_temp.c_cc[VSUSP] = -1;
		tc_temp.c_cc[VLNEXT] = -1;
		tc_temp.c_cc[VWERASE] = -1;
		tc_temp.c_cc[VREPRINT] = -1;
		tc_temp.c_cc[VDISCARD] = -1;
	}
	tcsetattr(0, TCSANOW, &tc_temp);
#endif
}

/* md_heed_signals():
 *
 * This routine tells the program to call particular routines when
 * certain interrupts/events occur:
 *
 *      SIGINT: call onintr() to interrupt fight with monster or long rest.
 *      SIGQUIT: call byebye() to check for game termination.
 *      SIGHUP: call error_save() to save game when terminal hangs up.
 *
 *		On VMS, SIGINT and SIGQUIT correspond to ^C and ^Y.
 *
 * This routine is not strictly necessary and can be stubbed.  This will
 * mean that the game cannot be interrupted properly with keyboard
 * input, this is not usually critical.
 */

void md_heed_signals(void)
{
#if 0
	signal(SIGINT, onintr);
	signal(SIGQUIT, byebye);
	signal(SIGHUP, error_save);
#endif
}

/* md_ignore_signals():
 *
 * This routine tells the program to completely ignore the events mentioned
 * in md_heed_signals() above.  The event handlers will later be turned on
 * by a future call to md_heed_signals(), so md_heed_signals() and
 * md_ignore_signals() need to work together.
 *
 * This function should be implemented or the user risks interrupting
 * critical sections of code, which could cause score file, or saved-game
 * file, corruption.
 */

void md_ignore_signals(void)
{
#if 0
	signal(SIGQUIT, SIG_IGN);
	signal(SIGINT, SIG_IGN);
	signal(SIGHUP, SIG_IGN);
	signal(SIGTSTP,SIG_IGN);	/* No ^Z ! */
#endif
}

/* md_get_file_id():
 *
 * This function returns an integer that uniquely identifies the specified
 * file.  It need not check for the file's existence.  In UNIX, the inode
 * number is used.
 *
 * This function need not be implemented.  To stub the routine, just make
 * it return 0.  This will make the game less able to prevent users from
 * modifying saved-game files.  This is probably no big deal.
 */

int md_get_file_id(char *fname)
{
#if 0
	struct stat sbuf;

	if (stat(fname, &sbuf))
	{
		return(-1);
	}
	return((int) sbuf.st_ino);
#endif
        return 0;
}

/* md_link_count():
 *
 * This routine returns the number of hard links to the specified file.
 *
 * This function is not strictly necessary.  On systems without hard links
 * this routine can be stubbed by just returning 1.
 */

int md_link_count(char *fname)
{
#if 0
	struct stat sbuf;

	stat(fname, &sbuf);
	return((int) sbuf.st_nlink);
#endif
        return 1;
}

/* md_gct(): (Get Current Time)
 *
 * This function returns the current year, month(1-12), day(1-31), hour(0-23),
 * minute(0-59), and second(0-59).  This is used for identifying the time
 * at which a game is saved.
 *
 * This function is not strictly necessary.  It can be stubbed by returing
 * zeros instead of the correct year, month, etc.  If your operating
 * system doesn't provide all of the time units requested here, then you
 * can provide only those that it does, and return zeros for the others.
 * If you cannot provide good time values, then users may be able to copy
 * saved-game files and play them.  
 */

void md_gct(struct rogue_time *rt_buf)
{
	// TODO
/*
	struct timeval tv;
	struct timezone tzp;
	struct tm *t;
	long seconds;

	gettimeofday(&tv, &tzp);
	seconds = (long) tv.tv_sec;
	t = localtime(&seconds);
*/
	rt_buf->year = 0;
	rt_buf->month = 0;
	rt_buf->day = 0;
	rt_buf->hour = 0;
	rt_buf->minute = 0;
	rt_buf->second = 0;
}

/* md_gfmt: (Get File Modification Time)
 *
 * This routine returns a file's date of last modification in the same format
 * as md_gct() above.
 *
 * This function is not strictly necessary.  It is used to see if saved-game
 * files have been modified since they were saved.  If you have stubbed the
 * routine md_gct() above by returning constant values, then you may do
 * exactly the same here.
 * Or if md_gct() is implemented correctly, but your system does not provide
 * file modification dates, you may return some date far in the past so
 * that the program will never know that a saved-game file being modified.  
 * You may also do this if you wish to be able to restore games from
 * saved-games that have been modified.
 */

void md_gfmt(char *fname, struct rogue_time *rt_buf)
{
	// TODO
/*
	struct stat sbuf;
	long seconds;
	struct tm *t;

	stat(fname, &sbuf);
	seconds = (long) sbuf.st_mtime;
	t = localtime(&seconds);
*/
	rt_buf->year = 0;
	rt_buf->month = 0;
	rt_buf->day = 0;
	rt_buf->hour = 0;
	rt_buf->minute = 0;
	rt_buf->second = 0;
}

/* md_df: (Delete File)
 *
 * This function deletes the specified file, and returns true (1) if the
 * operation was successful.  This is used to delete saved-game files
 * after restoring games from them.
 *
 * Again, this function is not strictly necessary, and can be stubbed
 * by simply returning 1.  In this case, saved-game files will not be
 * deleted and can be replayed.
 */

boolean md_df(char *fname)
{
	/* uncomment if you want saved game to be deleted
	if (unlink(fname))
	{
		return(0);
	}
	*/
	return(1);
}

int g_enter_name;
void end_name()
{
  if (strlen(g_keyboard_str)>2) {
    deactivate_sprites(0, 128);
    g_enter_name = 0;
  }
}

void update_name(void) {
  check_message();
  message(g_keyboard_str, 1);
}
/* md_gln: (Get login name)
 *
 * This routine returns the login name of the user.  This string is
 * used mainly for identifying users in score files.
 *
 * A dummy string may be returned if you are unable to implement this
 * function, but then the score file would only have one name in it.
 */

char * md_gln(void)
{
#if 0
	struct passwd *t;
	t = getpwuid(getuid());
	return(t->pw_name);
#endif
        /* Could add a virtual keyboard here to enter name */

        message("Enter your name!",1);
        load_vk('>');
        set_vk_callbacks(update_name, end_name, 0);
        g_enter_name = 1;
        while(g_enter_name) {
          virtual_keys();
          wait_vsync_int();
        }
        update_keys();
        return g_keyboard_str;
}

/* md_sleep:
 *
 * This routine causes the game to pause for the specified number of
 * seconds.
 *
 * This routine is not necessary at all, and can be stubbed with no ill
 * effects.
 */

void md_sleep(int nsecs)
{
  /* 60 frames per second */
  nsecs*=60;
  while (nsecs--) {
    wait_vsync_int();
  }
}

/* md_malloc()
 *
 * This routine allocates, and returns a pointer to, the specified number
 * of bytes.  This routines absolutely MUST be implemented for your
 * particular system or the program will not run at all.  Return zero
 * when no more memory can be allocated.
 */

char * md_malloc(int n)
{
	char *t;

	t = malloc(n);
	return(t);
}

/* md_gseed() (Get Seed)
 *
 * This function returns a seed for the random number generator (RNG).  This
 * seed causes the RNG to begin generating numbers at some point in it's
 * sequence.  Without a random seed, the RNG will generate the same set
 * of numbers, and every game will start out exactly the same way.  A good
 * number to use is the process id, given by getpid() on most UNIX systems.
 *
 * You need to find some single random integer, such as:
 *   process id.
 *   current time (minutes + seconds) returned from md_gct(), if implemented.
 *   
 * It will not help to return "get_rand()" or "rand()" or the return value of
 * any pseudo-RNG.  If you cannot a random number, you can just return 1,
 * but this means you games will ALWAYS start the same way, and will play
 * exactly the same way given the same input.
 */

int md_gseed(void)
{
#if 1
  extern int game_frames;
  check_message();
  message("Press A!", 1);
  wait_for_ack();
  messageclrtoeol();
  return game_frames;
#else
	return(1);
#endif
}

/* md_exit():
 *
 * This function causes the program to discontinue execution and exit.
 * This function must be implemented or the program will continue to
 * hang when it should quit.
 */

void md_exit(int status)
{
//	exit(status);
  //reset_gba();
#if 1
  *(u16 *)0x03007ffa = 0;  /* reset to ROM (= 0) rather than RAM (= 1) */
  asm volatile(
    "mov r0, #0xfc \n"  /* clear everything other than RAM */
#ifdef __thumb__
    "swi 1  \n"
    "swi 0  \n"
#else
    "swi 1 << 16 \n"
    "swi 0 << 16 \n"
#endif
    ::: "r0", "r1", "r2", "r3");
#endif
}

#define SAVE_OFFSET  4
// skip magic cookie
// skip magic cookie
#define SCORE_START     (SAVE_OFFSET + 3)
#define SCORE_MAX_SIZE  (10*82 + 10*30)
#define SAVE_HEADER_SIZE  ( 4 )
#define SAVE_START   (SAVE_OFFSET + 3 + SCORE_START + SCORE_MAX_SIZE+ SAVE_HEADER_SIZE)
#define SAVE_MAX     (16000)
int has_save_game() {
  char magic[4];
  int i;
  for (i = 0; i < 4; i++)
    magic[i] = LoadByte(SAVE_OFFSET + i);
  magic[3] = 0;
  if (strcmp(magic,"GBR"))
    return 0;
  return 1;
}

int load_size(int offset)
{
  int size = LoadByte(offset-SAVE_HEADER_SIZE);
  size |= LoadByte(offset-SAVE_HEADER_SIZE+1)<<8;
  size |= LoadByte(offset-SAVE_HEADER_SIZE+2)<<16;
  size |= LoadByte(offset-SAVE_HEADER_SIZE+3)<<24;
  return size;
}

void save_size(int offset, int size)
{
      SaveByte(offset-SAVE_HEADER_SIZE,     size&0xff);
      SaveByte(offset-SAVE_HEADER_SIZE+1,   (size>>8)&0xff);
      SaveByte(offset-SAVE_HEADER_SIZE+2,   (size>>16)&0xff);
      SaveByte(offset-SAVE_HEADER_SIZE+3,   (size>>24)&0xff);
}

md_file_t * md_fopen(const char * name, const char * mode) {
  /* return the "file" pointer - if read then check save actually belongs to us */
  /*
   * hackily check for score_file name - anything else must be 'save game' data 
   * score_file has a max size - the first time read size should be zero'd
   * */
  //gbaprintf("%s\n", __PRETTY_FUNCTION__);
  int start = SCORE_START;
  int max  = SCORE_MAX_SIZE;
  if (mode[0] == 'r') {
    /* do we have a save game here? 
     * if not then return 0 - nothing to read
     * otherwise return the pointer to read
     */
    if (!has_save_game()) {
      return 0;
    }
  } else if (mode[0] == 'w' || mode[1]=='+') {
    // writing
    if (!has_save_game())
    {
      //gbaprintf("Saving magic cookie...\n");
      // create magic cookie
      SaveByte(SAVE_OFFSET,   (unsigned char)'G');
      SaveByte(SAVE_OFFSET+1, (unsigned char)'B');
      SaveByte(SAVE_OFFSET+2, (unsigned char)'R');
      /* check if we are loading save game or score file */
      save_size(SCORE_START+SAVE_HEADER_SIZE, 0);
      save_size(SAVE_START+SAVE_HEADER_SIZE, 0);
      if (strcmp(score_file,name)!=0) {
        /* load from save game file offset */
        start = SAVE_START;
        max = SAVE_MAX;
      }
    }
  }
  md_file_t * fp = (md_file_t * ) malloc(sizeof(md_file_t));
  fp->start = start+SAVE_HEADER_SIZE;
  fp->position = fp->start;
  fp->mode[0] = mode[0];
  fp->mode[1] = mode[1];

  
  fp->size = load_size(fp->start);
  fp->max_size = max;
  
  return fp;
}
size_t md_fread(void *ptr, size_t size, size_t nmemb, md_file_t *stream) {
  //gbaprintf("%s\n", __PRETTY_FUNCTION__);
  if (!stream)
    return 0;
  if (stream->mode[0] != 'r' && stream->mode[1] != '+')
  {
    return 0;
  }
  size_t i;
  size_t total = nmemb * size;
  u8 * out = (u8*)ptr;
  int position = stream->position;
  int max_pos  = stream->start + stream->size;
  for (i = 0; i < total && position < max_pos; i++) {
//    gbaprintf("Reading value %d:  %c\n", position, LoadByte(position));
    *out++ = LoadByte(position++);
  }
  stream->position = position;
  //gbaprintf("Read %d bytes\n", i);
  return i;
}
size_t md_fwrite(const void *ptr, size_t size, size_t nmemb, md_file_t *stream)
{
  //gbaprintf("%s\n", __PRETTY_FUNCTION__);
  if (!stream)
    return 0;
  if (stream->mode[0] != 'w' && stream->mode[1] != '+')
    return 0;
  size_t i;
  size_t total = nmemb * size;
  const u8 * data = (const u8*)ptr;
  int position = stream->position;
  int max_pos  = stream->start + stream->max_size;
  for (i = 0; i < total && position < max_pos; i++) {
    //gbaprintf("Writing value %d:  %c\n", position, *data);
    SaveByte(position++, *data++);
  }
  //SaveByte(position++, 0xff);
  stream->position = position;

  if ( (stream->position - stream->start) > stream->size) 
  {
    // update size...
    stream->size = stream->position - stream->start;
    save_size(stream->start, stream->size);
  }
  
  //gbaprintf("Wrote %d bytes\n", i);
  return i;
}
int md_fclose(md_file_t * stream)
{
  //gbaprintf("%s\n", __PRETTY_FUNCTION__);
  /* would normally just call fclose, but here, no need to do anything */
  free(stream);
  stream = 0;
  return 0;
}

void md_rewind(md_file_t * stream)
{
  //gbaprintf("%s\n", __PRETTY_FUNCTION__);
  if (stream) {
    stream->position = stream->start;
  }
}
