/*
 * score.c
 *
 * This source herein may be modified and/or distributed by anybody who
 * so desires, with the following restrictions:
 *    1.)  No portion of this notice shall be removed.
 *    2.)  Credit shall not be taken for the creation of this source.
 *    3.)  This code is not to be traded, sold, or used for personal
 *         gain or profit.
 *
 */
/**
 * GBA version of scores.
 */

#include <agbprint.h>
#include "curses.h"
#include <stdio.h>
#include <string.h>
#include "rogue.h"
#include "hit.h"
#include "init.h"
#include "inventory.h"
#include "machdep.h"
#include "main.h"
#include "message.h"
#include "pack.h"
#include "ring.h"
#include "score.h"

const char * const score_file = ".roguescores";

extern char login_name[];
extern char *m_names[];
extern short max_level;
extern boolean score_only, show_skull, msg_cleared;
extern char *byebye_string, *nick_name;
extern fighter rogue;
extern struct id id_scrolls[];
extern struct id id_potions[];
extern struct id id_wands[];
extern struct id id_rings[];
extern struct id id_weapons[];
extern struct id id_armors[];

void center(short, char *);
void id_all(void);
void sell_pack(void);
void sf_error(void);
int name_cmp(char *, char *);
void insert_score(char [][82], int, int,
                  object *, int);
void nickize(char *, char *, char *);
int get_value(object *);

extern void vw_print_str_n(const char * const text, int row, int col, int len);

void killed_by(object *monster, int other)
{
	char buf[80];

	md_ignore_signals();

	if (other != QUIT)
	{
		rogue.gold = ((rogue.gold * 9) / 10);
	}

	if (other)
	{
		switch(other)
		{
		case HYPOTHERMIA:
			(void) strcpy(buf, "Died of hypothermia");
			break;
		case STARVATION:
			(void) strcpy(buf, "Died of starvation");
			break;
		case POISON_DART:
			(void) strcpy(buf, "Killed by a dart");
			break;
		case QUIT:
			(void) strcpy(buf, "Quit");
			break;
		}
	}
	else
	{
		(void) strcpy(buf, "Killed by ");
		if (is_vowel(m_names[monster->m_char - 'A'][0]))
		{
			(void) strcat(buf, "an ");
		}
		else
		{
			(void) strcat(buf, "a ");
		}
		(void) strcat(buf, m_names[monster->m_char - 'A']);
	}
	(void) strcat(buf, " with ");
	siprintf(buf+strlen(buf), "%ld gold", rogue.gold);
	if ((!other) && show_skull)
	{
		clear();
#ifdef __thumb__
          /* load an image or something... */
#else
                mvaddstr(4, 32, "__---------__");
                mvaddstr(5, 30, "_~             ~_");
		mvaddstr(6, 29, "/                 \\");
		mvaddstr(7, 28, "~                   ~");
		mvaddstr(8, 27, "/                     \\");
		mvaddstr(9, 27, "|    XXXX     XXXX    |");
		mvaddstr(10, 27, "|    XXXX     XXXX    |");
		mvaddstr(11, 27, "|    XXX       XXX    |");
		mvaddstr(12, 28, "\\         @         /");
		mvaddstr(13, 29, "--\\     @@@     /--");
		mvaddstr(14, 30, "| |    @@@    | |");
		mvaddstr(15, 30, "| |           | |");
		mvaddstr(16, 30, "| vvVvvvvvvvVvv |");
		mvaddstr(17, 30, "|  ^^^^^^^^^^^  |");
		mvaddstr(18, 31, "\\_           _/");
		mvaddstr(19, 33, "~---------~");
		center(21, (nick_name[0] ? nick_name : login_name));
		center(22, buf);
#endif
#ifdef __thumb__
                center_screen();
#endif
	}
	else
	{
		message(buf, 0);
	}
	message("", 0);
	put_scores(monster, other);
}

void win(void)
{
	unwield(rogue.weapon);		/* disarm and relax */
	unwear(rogue.armor);
	un_put_on(rogue.left_ring);
	un_put_on(rogue.right_ring);

	clear();
#ifdef __thumb__
        /* load an image or something... */
        vw_print_str_n("Congratulations, you have been admitted ",0,2,57-17);
        vw_print_str_n("to the Fighters' Guild. You return home,",0,3,57-17);
        vw_print_str_n("sell all your treasures at great profit ",0,4,57-17);
        vw_print_str_n("and retire into comfort.                ",0,5,57-17);

#else
	mvaddstr(10, 11, "@   @  @@@   @   @      @  @  @   @@@   @   @   @");
	mvaddstr(11, 11, " @ @  @   @  @   @      @  @  @  @   @  @@  @   @");
	mvaddstr(12, 11, "  @   @   @  @   @      @  @  @  @   @  @ @ @   @");
	mvaddstr(13, 11, "  @   @   @  @   @      @  @  @  @   @  @  @@    ");
	mvaddstr(14, 11, "  @    @@@    @@@        @@ @@    @@@   @   @   @");
	mvaddstr(17, 11, "Congratulations,  you have  been admitted  to  the");
	mvaddstr(18, 11, "Fighters' Guild.   You return home,  sell all your");
	mvaddstr(19, 11, "treasures at great profit and retire into comfort.");
#endif
	message("", 0);
	message("", 0);
	//id_all();
	//sell_pack();
        clear_text();
	put_scores((object *) 0, WIN);
}

void quit(boolean from_intrpt)
{
	char buf[128];
	int i;
	int	orow = 0;
	int ocol = 0;
	boolean mc = 0;

	md_ignore_signals();

	if (from_intrpt)
	{
		orow = rogue.row;
		ocol = rogue.col;
		mc = msg_cleared;

		for (i = 0; i < DCOLS; i++)
		{
			buf[i] = mvinch(0, i);
		}
	}
	check_message();
	message("Really quit? (y/n)", 1);
	if (rgetchar() != 'y')
	{
		md_heed_signals();
		check_message();
		if (from_intrpt)
		{
			for (i = 0; i < DCOLS; i++)
			{
				mvaddch(0, i, buf[i]);
			}
			msg_cleared = mc;
			move(orow, ocol);
			refresh();
		}
		return;
	}
	if (from_intrpt)
	{
		clean_up(byebye_string);
	}
	check_message();
	killed_by((object *) 0, QUIT);
}

#define TOP_RANKS   5
#define NAME_OFFSET  13
void put_scores(object *monster, int other)
{
  int i;
  int rank = TOP_RANKS;
  int found_player = -1;
  int n;
  int x;
  int ne = 0;
  char scores[TOP_RANKS][82];
  /*char buf[100];*/
  md_file_t *fp;
  long s;
  boolean failed = 0;
  char *mode = "r+w";

  turn_into_games();

  while ((fp = md_fopen(score_file, mode)) == NULL)
  {
    if (!failed)
    {
      mode = "w";
    }
    else
    {
      message("Cannot read/write/create score file.", 0);
      sf_error();
    }
    failed = 1;
  }
  turn_into_user();
  (void) xxx(1);

  for (i = 0; i < TOP_RANKS; i++)
  {
    if (((n = md_fread(scores[i], sizeof(char), 80, fp)) < 80) && (n != 0))
    {
      sf_error();
    } else if (n != 0) {
    } else {
      break;
    }
    ne++;
    if (!score_only)
    {
      if (!name_cmp(scores[i]+NAME_OFFSET, login_name))
      {
        x = 5;
        while (scores[i][x] == ' ')
        {
          x++;
        }
        s = lget_number(scores[i] + x);
        if (rogue.gold < s)
        {
          score_only = 1;
        }
        else
        {
          found_player = i;
        }
      }
    }
  }
  if (found_player != -1)
  {
    ne--;
    for (i = found_player; i < ne; i++)
    {
      (void) strcpy(scores[i], scores[i+1]);
    }
  }
  if (!score_only)
  {
    for (i = 0; i < ne; i++)
    {
      x = 5;
      while (scores[i][x] == ' ')
      {
        x++;
      }
      s = lget_number(scores[i] + x);

      if (rogue.gold >= s)
      {
        rank = i;
        break;
      }
    }
    if (ne == 0)
    {
      rank = 0;
    }
    else
    {
      if ((ne < TOP_RANKS) && (rank == TOP_RANKS))
      {
        rank = ne;
      }
    }
    if (rank < TOP_RANKS)
    {
      insert_score(scores, rank, (short) ne, monster, other);
      if (ne < TOP_RANKS)
      {
        ne++;
      }
    }
    md_rewind(fp);
  }

  clear();
  /*mvaddstr(3, 30, "Top  Ten  Rogueists");
  mvaddstr(8, 0, "Rank   Score   Name");*/

  vw_print_str_n( "Top  Five  Rogueists",3,1,19 );
  vw_print_str_n( "Rank  Score  Name",0, 2,19);

  md_ignore_signals();

  (void) xxx(1);

  for (i = 0; i < ne; i++)
  {
    if (i == rank)
    {
      standout();
    }
    if (i == 9)
    {
      scores[i][0] = '1';
      scores[i][1] = '0';
    }
    else
    {
      scores[i][0] = ' ';
      scores[i][1] = i + '1';
    }
    //mvaddstr(i+TOP_RANKS, 0, scores[i]);
    char *newline = strchr(scores[i],'\n');
    vw_print_str_n(scores[i],0,(i*2)+3,newline-scores[i]);
    vw_print_str_n(newline,0,(i*2)+4,strlen(newline));
    if (rank < TOP_RANKS)
    {
      md_fwrite(scores[i], sizeof(char), 80, fp);
    }
    if (i == rank)
    {
      standend();
    }
  }
  refresh();
  md_fclose(fp);
  message("", 0);
  clean_up("");
}

void insert_score(char scores[][82], int rank, int n, object *monster, int other)
{
	int i;
	char buf[82];

	if (n > 0)
	{
		for (i = n; i > rank; i--)
		{
			if ((i < TOP_RANKS) && (i > 0))
			{
				(void) strcpy(scores[i], scores[i - 1]);
			}
		}
	}
	siprintf(buf, "%2d %6d    %s\n", rank + 1, (int) rogue.gold, login_name); // (int) by me

	if (other)
	{
		switch(other)
		{
		case HYPOTHERMIA:
			(void) strcat(buf, "died of hypothermia");
			break;
		case STARVATION:
			(void) strcat(buf, "died of starvation");
			break;
		case POISON_DART:
			(void) strcat(buf, "killed by a dart");
			break;
		case QUIT:
			(void) strcat(buf, "quit");
			break;
		case WIN:
			(void) strcat(buf, "a total winner");
			break;
		}
	}
	else
	{
		(void) strcat(buf, "killed by ");
		if (is_vowel(m_names[monster->m_char - 'A'][0]))
		{
			(void) strcat(buf, "an ");
		}
		else
		{
			(void) strcat(buf, "a ");
		}
		(void) strcat(buf, m_names[monster->m_char - 'A']);
	}
	siprintf(buf + strlen(buf), " on level %d ",  max_level);
	if ((other != WIN) && has_amulet())
	{
		(void) strcat(buf, "with amulet");
	}
	for (i = strlen(buf); i < 79; i++)
	{
		buf[i] = ' ';
	}
	buf[79] = 0;
	(void) strcpy(scores[rank], buf);
}

int is_vowel(char ch)
{
	return( (ch == 'a') || (ch == 'e') || (ch == 'i')
	        || (ch == 'o') || (ch == 'u') );
}

void sell_pack(void)
{
	object *obj;
	short row = 2, val;
	char buf[80];

	obj = rogue.pack.next_object;

	clear();
	mvaddstr(1, 0, "Value      Item");

	while (obj)
	{
		if (obj->what_is != FOOD)
		{
			obj->identified = 1;
			val = get_value(obj);
			rogue.gold += val;

			if (row < DROWS)
			{
				siprintf(buf, "%5d      ", val);
				get_desc(obj, buf+11);
				mvaddstr(row++, 0, buf);
			}
		}
		obj = obj->next_object;
	}
	refresh();
	if (rogue.gold > MAX_GOLD)
	{
		rogue.gold = MAX_GOLD;
	}
	message("", 0);
}

int get_value(object *obj)
{
	int wc;
	int val = 0;

	wc = obj->which_kind;

	switch(obj->what_is)
	{
	case WEAPON:
		val = id_weapons[wc].value;
		if ((wc == ARROW) || (wc == DAGGER) || (wc == SHURIKEN)
		    || (wc == DART))
		{
			val *= obj->quantity;
		}
		val += (obj->d_enchant * 85);
		val += (obj->hit_enchant * 85);
		break;
	case ARMOR:
		val = id_armors[wc].value;
		val += (obj->d_enchant * 75);
		if (obj->is_protected)
		{
			val += 200;
		}
		break;
	case WAND:
		val = id_wands[wc].value * (obj->oclass + 1);
		break;
	case SCROLL:
		val = id_scrolls[wc].value * obj->quantity;
		break;
	case POTION:
		val = id_potions[wc].value * obj->quantity;
		break;
	case AMULET:
		val = 5000;
		break;
	case RING:
		val = id_rings[wc].value * (obj->oclass + 1);
		break;
	}
	if (val <= 0)
	{
		val = 10;
	}
	return(val);
}

void id_all(void)
{
	int i;

	for (i = 0; i < SCROLLS; i++)
	{
		id_scrolls[i].id_status = IDENTIFIED;
	}
	for (i = 0; i < WEAPONS; i++)
	{
		id_weapons[i].id_status = IDENTIFIED;
	}
	for (i = 0; i < ARMORS; i++)
	{
		id_armors[i].id_status = IDENTIFIED;
	}
	for (i = 0; i < WANDS; i++)
	{
		id_wands[i].id_status = IDENTIFIED;
	}
	for (i = 0; i < POTIONS; i++)
	{
		id_potions[i].id_status = IDENTIFIED;
	}
}

int name_cmp(char *s1, char *s2)
{
	short i = 0;
	int r;

	while(s1[i] != '\n')
	{
		i++;
	}
	s1[i] = 0;
	r = strcmp(s1, s2);
	s1[i] = '\n';
	return(r);
}

void xxxx(char *buf, int n)
{
	int i;
	unsigned char c;

	for (i = 0; i < n; i++)
	{
		/* It does not matter if accuracy is lost during this assignment */
		c = (unsigned char) xxx(0);

		buf[i] ^= c;
	}
}

long xxx(boolean st)
{
	static long f, s;
	long r;

	if (st)
	{
		f = 37;
		s = 7;
		return(0L);
	}
	r = ((f * s) + 9337) % 8887;
	f = s;
	s = r;
	return(r);
}

void nickize(char *buf, char *score, char *n_name)
{
	int i = 15;
	int j;

	while (score[i] != ':')
	{
		i++;
	}

	(void) strcpy(buf + 15, n_name);
	j = strlen(buf);

	while (score[i])
	{
		buf[j++] = score[i++];
	}
	buf[j] = 0;
	buf[79] = 0;
}

void center(short row, char *buf)
{
	short margin;

	margin = ((DCOLS - strlen(buf)) / 2);
	mvaddstr(row, margin, buf);
}

void sf_error(void)
{
	message("", 1);
	clean_up("sorry, score file is out of order");
}
